//
// Created by Peter Stanko on 15.2.16.
//

#ifndef DYNAMIC_LIB_H
#define DYNAMIC_LIB_H

#include <stddef.h>

/**
 * Copies string and returns pointer to new string
 * @param str[in] - original string
 * @return New string allocated by malloc(3)
 */
char *dyn_strcpy(const char *str);

/**
 * Joins both strings into one
 * @param pre[in] - First string
 * @param post[in] - Second string
 * @return New string
 */
char *dyn_strjoin(const char *pre, const char *post);

/**
 * Reads full line from standard input (stdin)
 * @return Whole line
 */
char *read_line(void);

/**
 * Allocates 2D Array for provided number of rows and cols
 *
 * @param[in] rows Number of rows for 2D array
 * @param[in] row_sizes Length of the columns for each row
 */
void **dyn_alloc2d(size_t rows, const size_t row_sizes[rows]);

/**
 * Frees Memory for 2D array
 *
 * @param[in] memory - memory allocated using `dyn_alloc2d()`
 * @param[in] rows - number of rows
 */
int dyn_free2d(void **memory, size_t rows);

/* Bonuses */

/**
 * Compuate the Pascal triangle and return matrix filled with values
 *
 * @param[in] depth Depth of the triangle
 */
unsigned **pascal_triangle(size_t depth);

/**
 * Implementation of dynamic split
 */
char **string_split(const char *orig, const char *splitter, size_t *size);

/**
 * Implementation of replace all.
 */
char *string_replace_all(const char *original, const char *from, const char *to);

#endif // DYNAMIC_LIB_H
